# pyground. A collection of utility and wrapper classes
# Copyright (C) 2004 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of pyground.
#
# pyground is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""A SAX Consumers collection.

[FIXME]
"""

class SAXConsumer(object):
    """A SAX events consumer capable object.

    The SAXConsumer object is the interface to be implemented by each object
    willing to consume XML elements from a SAX parser.
    """
    def __init__(self):
        if self.__class__ is SAXConsumer:
            lMessage = 'class %s is abstract' % str(self.__class__)
            logging.getLogger('ground.xml.parsers').error(lMessage)
            raise NotImplementedError(lMessage)


    def consumeElement(self, aName, aQName, aAttributes):
        """Notify a new XML element (a new opening tag).

        The aName argument is a tuple with the namespace and the name of
        the new XML element. aQName is the fully qualified name of this
        element, that is the name as it appeared in the stream (or the file).
        The aAttributes argument is an instance of AttributesNS. To keep
        a local reference on this object, it is necessary to invoke its
        method copy().

        (see www.python.org/doc/current/lib/content-handler-objects.html).
        """
        lMessage = "class %s misses 'consumeElement'" % str(self.__class__)
        logging.getLogger('ground.xml.parsers').error(lMessage)
        raise NotImplementedError(lMessage)


    def consumeEndElement(self, aName, aQName):
        """Notify a new closing tag.

        The aName argument is a tuple with the namespace and the name of
        the new XML element. aQName is the fully qualified name of this
        element, that is the name as it appeard in the flow (or the file).

        Unlike consumeElement which has to be defined, consumeEndElement
        is implemented and does nothing by default.
        """
        pass


    def consumeCharacters(self, aContent):
        """Notify a new PCDATA field.
        """
        lMessage = "class %s misses 'consumeCharacters'" % str(self.__class__)
        logging.getLogger('ground.xml.parsers').error(lMessage)
        raise NotImplementedError(lMessage)



class RootSAXConsumer(SAXConsumer):
    """A SAXConsumer object capable of self-building from one XML element.
    """
    def __init__(self):
        if self.__class__ is RootSAXConsumer:
            lMessage = 'class %s is abstract' % str(self.__class__)
            logging.getLogger('ground.xml.parsers').error(lMessage)
            raise NotImplementedError(lMessage)


    def buildFromElement(self, aName, aQName, aAttributes):
        """Initialize this object with the given XML element.
        """
        lMessage = "class %s misses 'buildFromElement'"% str(self.__class__)
        logging.getLogger('ground.xml.parsers').error(lMessage)
        raise NotImplementedError(lMessage)



class Element2Field(SAXConsumer):
    """A SAXConsumer object translating XML elements into class attributes.
    """
    def __init__(self, aSetter):
        """Constructor.

        The aSetter callable will be used upon occurence of an element.
        aSetter must accept one argument (or two and be bound, so something
        like setAttribute(self, aValue) in a class is correct).
        """
        if aSetter == None:
            lMessage = 'aSetter is None in %s ' % str(self.__class__)
            logging.getLogger('ground.xml.parsers').error(lMessage)
            raise AttributeError(lMessage)
        self.__setter = aSetter


    def getSetter(self):
        """Return the callable used to register the attribute upon occurence
        of an XML element.
        """
        return self.__setter


    def consumeElement(self, aName, aQName, aAttributes):
        """Notify a new XML element (a new opening tag).

        This object shouldn't consume any element. An exception is
        systematically raised.
        """
        lMessage = "%s cannot consume element %s" % (str(self.__class__), aName)
        logging.getLogger('ground.xml.parsers').error(lMessage)
        raise AttributeError(lMessage)


    def consumeEndElement(self, aName, aQName):
        """Notify a new closing tag.

        This object shouldn't consume any element. An exception is
        systematically raised.
        """
        lMessage = "%s cannot consume end tag %s" % (str(self.__class__), aName)
        logging.getLogger('ground.xml.parsers').error(lMessage)
        raise AttributeError(lMessage)


    def consumeCharacters(self, aContent):
        """Notify a new PCDATA field.

        This data is transmitted to the object who asked for it by giving
        the right setter method to this object constructor.
        """
        self.__setter(aContent)
