# pyground. A collection of utility and wrapper classes
# Copyright (C) 2004 David Soulayrol <dsoulayrol@free.fr>
#
# This file is part of pyground.
#
# pyground is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""A SAX parsers collection.

{TODO: to be wrote. insert example in __init__.py]
[TODO: talk about ground.xml.parsers logger]
"""

import logging, urllib2, xml.sax

from xml.sax.handler import ContentHandler, DTDHandler, ErrorHandler
from xml.sax.handler import EntityResolver, feature_validation
from xml.sax.xmlreader import XMLReader, InputSource


class XmlSAXParser(object,
                   ContentHandler,
                   DTDHandler,
                   ErrorHandler,
                   EntityResolver):
    """Base class of an XML stream analyser using SAX2 API.
    """
    
    def __init__(self, aReportEndElements = False):
        """Constructor.

        The analyser doesn't notify closing tags by default. Set
        aReportEndElements to True to do so.
        """
        if self.__class__ is XmlSAXParser:
            lMessage = 'class %s is abstract'% str(self.__class__)
            logging.getLogger('ground.xml.parsers').error(lMessage)
            raise NotImplementedError(lMessage)

        self.__xmlReader = xml.sax.make_parser()
        self.__initialize()
        self.__reportEndElements = aReportEndElements
        

    def __initialize(self):
        self.__stack = []
        self.__characterStack = []
        self.__buffer = ''
        self.__consumer = None
        
        
    def getStack(self):
        return self.__stack


    def getCharacterStack(self):
        return self.__characterStack


    def getBuffer(self):
        return self.__buffer

    
    def getConsumer(self):
        return self.__consumer


    def parse(self, aUrl):
        """Initialize the parsing on the file designed by the given URL.

        Before beginning a new parsing, XML elements and characters stack
        are cleared, so as the Consumer object.
        """
        try:
            lStream = urllib2.urlopen(aUrl)
            self.parse(lStream)
            lStream.close()
            
        except urllib2.URLError, aMessage:
            logging.getLogger('ground.xml.parsers').error(aMessage)
            raise urllib2.URLError(aMessage)
        
        return self.__consumer


    def parse(self, aStream):
        """Initialize the parsing on the given stream.

        Before beginning a new parsing, XML elements and characters stack
        are cleared, so as the Consumer object.
        """
        self.__initialize()
        
        try:
            self.__xmlReader.setFeature(feature_validation, False)
            self.__xmlReader.setContentHandler(self)
            self.__xmlReader.parse(InputSource(aStream))

        except Exception, aMessage:
            logging.getLogger('ground.xml.parsers').error(aMessage)
            raise Exception(aMessage)
        
        return self.__consumer


    def createConsumer(self, aName, aQName, aAttributes):
        """Cre l'objet Consumer.

        This method must be implemented in subclasses. It must return
        the created consumer.

        The aName argument is a tuple with the namespace and the name of
        the new XML element. aQName is the fully qualified name of this
        element, that is the name as it appeared in the stream (or the file).
        The aAttributes argument is an instance of AttributesNS. To keep
        a local reference on this object, it is necessary to invoke its
        method copy().

        (see www.python.org/doc/current/lib/content-handler-objects.html).
        """
        lMessage = "class %s misses 'createConsumer'"% str(self.__class__)
        logging.getLogger('ground.xml.parsers').error(lMessage)
        raise NotImplementedError(lMessage)

    
    def characters(self, aContent):
        """Notify new characters.
        """
        lStrippedContent = aContent.strip()
        if len(lStrippedContent) > 0:
            if self.__consumer != None:
                self.__buffer = self.__buffer.join(lStrippedContent)


    def startElement(self, aName, aAttributes):
        """Notify a new XML element.

        If no Consumer exists, a new one is created. Then, the current
        Consumer must manage this new element. 
        """
        self.startElementNS(aName, '', aAttributes)


    def startElementNS(self, aName, aQName, aAttributes):
        """Notify a new XML element.

        If no Consumer exists, a new one is created. Then, the current
        Consumer must manage this new element. 

        The aName argument is a tuple with the namespace and the name of
        the new XML element. aQName is the fully qualified name of this
        element, that is the name as it appeared in the stream (or the file).
        The aAttributes argument is an instance of AttributesNS. To keep
        a local reference on this object, it is necessary to invoke its
        method copy().

        (see www.python.org/doc/current/lib/content-handler-objects.html).
        """
        if self.__consumer != None:
            lConsumer = self.__consumer.consumeElement(aName, aQName, aAttributes)
            self.__stack.append(self.__consumer)
            self.__consumer = lConsumer
            self.__characterStack.append(self.__buffer)
            self.__buffer = ''

        else:
            self.__consumer = self.createConsumer(aName, aQName, aAttributes)
            self.__buffer = ''


    def endElement(self, aName):
        """Notify a new ending tag.

        The Consumer object at the top of the stack is suppressed.
        """
        self.endElementNS(aName, '')


    def endElementNS(self, aName, aQName):
        """Notify a new ending tag.

        The Consumer object at the top of the stack is suppressed.
        """
        if self.__consumer != None:
            lCharacters = self.__buffer.strip()
            if len(lCharacters) > 0:
                self.__consumer.consumeCharacters(lCharacters)
                
        if len(self.__stack) > 0:
            self.__buffer = self.__characterStack.pop()
            self.__consumer = self.__stack.pop()
            if self.__reportEndElements:
                self.__consumer.consumeEndElement(aName, aQName)


class XmlBootstrapParser(XmlSAXParser):
    """XML SAX Parser.
    """

    def __init__(self, aRootSAXConsumer, aReportEndElements = False):
        """Constructor.

        The XmlBootstrapParser parser works with a RootSAXConsumer object. Such
        a consumer is capable of initialize itself from an XML element.
        """
        XmlSAXParser.__init__(self, aReportEndElements)
        if aRootSAXConsumer == None:
            lMessage = 'SAX consumer is None in %s '% str(self.__class__)
            logging.getLogger('ground.xml.parsers').error(lMessage)
            raise AttributeError(lMessage)
        self.__consumer = aRootSAXConsumer


    def __initialize(self):
        self.__stack = []
        self.__characterStack = []
        self.__buffer = ''


    def createConsumer(self, aName, aQName, aAttributes):
        """Create the consumer object.

        see XmlSAXParser.createConsumer.
        """
        self.__consumer.buildFromElement(aName, aQName, aAttributes)
        return self.__consumer
    
